<?php
/***********************************************************************

  Copyright (C) 2002, 2003, 2004  Rickard Andersson (rickard@punbb.org)

  This file is part of PunBB.

  PunBB is free software; you can redistribute it and/or modify it
  under the terms of the GNU General Public License as published
  by the Free Software Foundation; either version 2 of the License,
  or (at your option) any later version.

  PunBB is distributed in the hope that it will be useful, but
  WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
  MA  02111-1307  USA

************************************************************************/


//
// Cookie stuff!
//
function check_cookie(&$cur_user)
{
	global $db, $pun_config, $pun_root, $cookie_name, $cookie_path, $cookie_domain, $cookie_secure;

	$now = time();
	$expire = $now + 31536000;	// The cookie expires after a year

	if (isset($_COOKIE[$cookie_name]))
	{
		list($cookie['username'], $cookie['password_hash']) = unserialize(unescape($_COOKIE[$cookie_name]));

		if (strcasecmp($cookie['username'], 'Guest'))
		{
			$result = $db->query('SELECT * FROM '.$db->prefix.'users WHERE username=\''.addslashes($cookie['username']).'\' AND password=\''.addslashes($cookie['password_hash']).'\'') or error('Unable to fetch user information', __FILE__, __LINE__, $db->error());
			$cur_user = $db->fetch_assoc($result);

			if ($cur_user['disp_topics'] == '')
				$cur_user['disp_topics'] = $pun_config['o_disp_topics_default'];
			if ($cur_user['disp_posts'] == '')
				$cur_user['disp_posts'] = $pun_config['o_disp_posts_default'];

			// Determine what style to use
			if (!@file_exists($pun_root.'style/'.$cur_user['style'].'.css'))
				$cur_user['style'] = $pun_config['o_default_style'];

			// If the user couldn't be found
			if (!isset($cur_user['id']))
			{
				setcookie($cookie_name, serialize(array('Guest', 'Guest')), $expire, $cookie_path, $cookie_domain, $cookie_secure);

				$cookie['username'] = 'Guest';
				$cookie['is_guest'] = true;

				return $cookie;
			}

			if ($cur_user['save_pass'] == '0')
				$expire = 0;

			// Define this if you don't want PunBB to update the last_action/last_visit data
			if (!defined('PUN_QUIET_VISIT'))
			{
				// Has the user been idle longer than o_timeout_visit?
				if ($now > ($cur_user['last_action'] + $pun_config['o_timeout_visit']))
				{
					$cur_user['last_visit'] = $cur_user['last_action'];
					$db->query('UPDATE '.$db->prefix.'users SET last_visit=last_action, last_action='.$now.' WHERE id='.$cur_user['id']) or error('Unable to update user visit data', __FILE__, __LINE__, $db->error());
				}
				else
					$db->query('UPDATE '.$db->prefix.'users SET last_action='.$now.' WHERE id='.$cur_user['id']) or error('Unable to update user visit data', __FILE__, __LINE__, $db->error());

				$cur_user['last_action'] = $now;

				setcookie($cookie_name, serialize(array($cur_user['username'], $cookie['password_hash'])), $expire, $cookie_path, $cookie_domain, $cookie_secure);
			}

			$cookie['is_guest'] = false;
		}
		else
			$cookie['is_guest'] = true;
	}
	else
	{
		$cookie['username'] = 'Guest';
		$cookie['is_guest'] = true;
	}

	return $cookie;
}


//
// Check whether the connecting user is banned (and delete any expired bans while we're at it)
//
function check_bans()
{
	global $db, $pun_config, $lang_common, $cookie, $cur_user;

	// Admins aren't affected
	if ($cur_user['status'] == PUN_ADMIN)
		return;

	// Add a dot at the end of the IP address to prevent banned address 192.168.0.5 from matching e.g. 192.168.0.50
	$user_ip = get_remote_address().'.';

	$result = $db->query('SELECT id, username, ip, message, expire FROM '.$db->prefix.'bans WHERE username IS NOT NULL OR ip IS NOT NULL OR expire IS NOT NULL') or error('Unable to fetch ban list', __FILE__, __LINE__, $db->error());

	while ($cur_ban = $db->fetch_assoc($result))
	{
		// Has this ban expired?
		if ($cur_ban['expire'] != '' && $cur_ban['expire'] <= time())
		{
			$db->query('DELETE FROM '.$db->prefix.'bans WHERE id='.$cur_ban['id']) or error('Unable to delete expired ban', __FILE__, __LINE__, $db->error());
			continue;
		}

		if ($cur_ban['username'] != '' && !strcasecmp($cookie['username'], $cur_ban['username']))
			message($lang_common['Ban message'].' '.(($cur_ban['expire'] != '') ? $lang_common['Ban message 2'].' '.format_time($cur_ban['expire'], true).'. ' : '').(($cur_ban['message'] != '') ? $lang_common['Ban message 3'].'<br><br><b>'.$cur_ban['message'].'</b><br><br>' : '<br><br>').$lang_common['Ban message 4'].' <a href="mailto:'.$pun_config['o_admin_email'].'">'.$pun_config['o_admin_email'].'</a>.', true);

		if ($cur_ban['ip'] != '')
		{
			$cur_ban_ips = explode(' ', $cur_ban['ip']);

			for ($i = 0; $i < count($cur_ban_ips); ++$i)
			{
				$cur_ban_ips[$i] = $cur_ban_ips[$i].'.';

				if (substr($user_ip, 0, strlen($cur_ban_ips[$i])) == $cur_ban_ips[$i])
					message($lang_common['Ban message'].' '.(($cur_ban['expire'] != '') ? $lang_common['Ban message 2'].' '.format_time($cur_ban['expire'], true).'. ' : '').(($cur_ban['message'] != '') ? $lang_common['Ban message 3'].'<br><br><b>'.$cur_ban['message'].'</b><br><br>' : '<br><br>').$lang_common['Ban message 4'].' <a href="mailto:'.$pun_config['o_admin_email'].'">'.$pun_config['o_admin_email'].'</a>.', true);
			}
		}
	}
}


//
// Update "Users online"
//
function update_users_online()
{
	global $db, $pun_config, $cookie, $cur_user;

	if (!$cookie['is_guest'])
	{
		$user_id = $cur_user['id'];
		$ident = addslashes($cookie['username']);
	}
	else
	{
		$user_id = 0;
		$ident = get_remote_address();
	}

	$now = time();

	// Delete entries older than timeout_online seconds and any duplicates (start transaction)
	$db->query('DELETE FROM '.$db->prefix.'online WHERE logged<'.($now-$pun_config['o_timeout_online']).' OR ident=\''.addslashes($cookie['username']).'\' OR ident=\''.get_remote_address().'\'', PUN_TRANS_START) or error('Unable to delete from online list', __FILE__, __LINE__, $db->error());

	// Add a new entry. username and user_id if logged in; ip and user_id=0 if not (end transaction)
	$db->query('INSERT INTO '.$db->prefix.'online (user_id, ident, logged) VALUES(\''.$user_id.'\', \''.$ident.'\', '.$now.')', PUN_TRANS_END) or error('Unable to insert into online list', __FILE__, __LINE__, $db->error());
}


//
// Generate the "navigator" that appears at the top of every page
//
function generate_navlinks()
{
	global $pun_config, $lang_common, $cookie, $cur_user;

	// Home and Userlist should always be displayed
	$links[] = '<a href="index.php">'.$lang_common['Home'].'</a> | <a href="userlist.php">'.$lang_common['User list'].'</a>';

	if ($pun_config['o_rules'] == '1')
		$links[] = '<a href="misc.php?action=rules">'.$lang_common['Rules'].'</a>';

	if ($cookie['is_guest'])
	{
		if ($pun_config['o_search'] == '1' && $pun_config['p_guests_search'] == '1')
			$links[] = '<a href="search.php">'.$lang_common['Search'].'</a>';

		$links[] = '<a href="register.php">'.$lang_common['Register'].'</a> | <a href="login.php">'.$lang_common['Login'].'</a>';
		
		$info = $lang_common['Not logged in'];
	}
	else
	{
		if ($cur_user['status'] < PUN_MOD)
		{
			if ($pun_config['o_search'] == '1')
				$links[] = '<a href="search.php">'.$lang_common['Search'].'</a>';

			$links[] = '<a href="profile.php?id='.$cur_user['id'].'">'.$lang_common['Profile'].'</a>';
			$links[] = '<a href="login.php?action=out">'.$lang_common['Logout'].'</a>';
		}
		else
		{
			$links[] = '<a href="search.php">'.$lang_common['Search'].'</a>';
			$links[] = '<a href="profile.php?id='.$cur_user['id'].'">'.$lang_common['Profile'].'</a>';
			$links[] = '<a href="admin_index.php">'.$lang_common['Admin'].'</a>';
			$links[] = '<a href="login.php?action=out">'.$lang_common['Logout'].'</a>';
		}
	}

	return implode(' | ', $links);
}


//
// Update posts, topics, last_post, last_post_id and last_poster for a forum (redirect topics are not included)
// If $transaction == PUN_TRANS_END, this function will end the current transaction
//
function update_forum($forum_id, $transaction = 0)
{
	global $db;

	$result = $db->query('SELECT COUNT(id), SUM(num_replies) FROM '.$db->prefix.'topics WHERE moved_to IS NULL AND forum_id='.$forum_id) or error('Unable to fetch forum topic count', __FILE__, __LINE__, $db->error());
	list($num_topics, $num_posts) = $db->fetch_row($result);

	$num_posts = $num_posts + $num_topics;		// $num_posts is only the sum of all replies (we have to add the topic posts)

	$result = $db->query('SELECT last_post, last_post_id, last_poster FROM '.$db->prefix.'topics WHERE forum_id='.$forum_id.' AND moved_to IS NULL ORDER BY last_post DESC LIMIT 1') or error('Unable to fetch last_post/last_post_id/last_poster', __FILE__, __LINE__, $db->error());
	if ($db->num_rows($result))		// There are topics in the forum
	{
		list($last_post, $last_post_id, $last_poster) = $db->fetch_row($result);

		$db->query('UPDATE '.$db->prefix.'forums SET num_topics='.$num_topics.', num_posts='.$num_posts.', last_post='.$last_post.', last_post_id='.$last_post_id.', last_poster=\''.addslashes($last_poster).'\' WHERE id='.$forum_id, $transaction) or error('Unable to update last_post/last_post_id/last_poster', __FILE__, __LINE__, $db->error());
	}
	else	// There are no topics
		$db->query('UPDATE '.$db->prefix.'forums SET num_topics=0, num_posts=0, last_post=NULL, last_post_id=NULL, last_poster=NULL WHERE id='.$forum_id, $transaction) or error('Unable to update last_post/last_post_id/last_poster', __FILE__, __LINE__, $db->error());
}


//
// Check whether the current user is an administrator or a moderator in $forum_id (also check if forum is closed and/or admmod_only)
//
function is_admmod($forum_id, &$forum_closed, &$admmod_only)
{
	global $db, $cur_user;

	$result = $db->query('SELECT moderators, admmod_only, closed FROM '.$db->prefix.'forums WHERE id='.$forum_id) or error('Unable to fetch forum info', __FILE__, __LINE__, $db->error());

	if ($db->num_rows($result))
	{
		list($moderators, $admmod_only, $forum_closed) = $db->fetch_row($result);
		$mods_array = ($moderators != '') ? unserialize($moderators) : array();

		return ($cur_user['status'] == PUN_ADMIN || ($cur_user['status'] == PUN_MOD && array_key_exists($cur_user['username'], $mods_array))) ? true : false;
	}
	else
		return false;
}


//
// Replace censored words in $text
//
function censor_words($text)
{
	global $db;
	static $search_for, $replace_with;

	// If not already built, build an array of censor words and their replacement text
	if (empty($search_for))
	{
		$result = $db->query('SELECT search_for, replace_with FROM '.$db->prefix.'censoring') or error('Unable to fetch censor word list', __FILE__, __LINE__, $db->error());
		$num_words = $db->num_rows($result);

		if ($num_words)
		{
			for ($i = 0; $i < $num_words; ++$i)
			{
				list($search_for[$i], $replace_with[$i]) = $db->fetch_row($result);
				$search_for[$i] = '/\b('.str_replace('\*', '\w*?', preg_quote($search_for[$i], '/')).')\b/i';
			}
		}
		else
			$search_for[] = 1;	// Dummy entry
	}

	if (!empty($search_for) && $search_for[0] != 1)
		$text = substr(preg_replace($search_for, $replace_with, ' '.$text.' '), 1, -1);

	return $text;
}


//
// Determines the correct title for $user
// $user must contain the elements 'username', 'title', 'status' and 'posts'
//
function get_title($user)
{
	global $db, $pun_config, $lang_common;
	static $ban_list, $rank_list;

	// If not already built, build an array of banned usernames
	if (empty($ban_list))
	{
		$ban_list[] = 1;		// Dummy entry

		$result = $db->query('SELECT LOWER(username) FROM '.$db->prefix.'bans WHERE username IS NOT NULL') or error('Unable to fetch banned username list', __FILE__, __LINE__, $db->error());

		while ($row = $db->fetch_row($result))
			$ban_list[] = $row[0];
	}

	// If not already built, build an array of ranks and their respective minimun number of posts
	if ($pun_config['o_ranks'] == '1' && empty($rank_list))
	{
		$rank_list[] = 1;	// Dummy entry

		$result = $db->query('SELECT rank, min_posts FROM '.$db->prefix.'ranks ORDER BY min_posts') or error('Unable to fetch rank list', __FILE__, __LINE__, $db->error());

		while ($row = $db->fetch_row($result))
			$rank_list[] = $row;
	}

	// If the user has a custom title
	if ($user['title'] != '')
		$user_title = pun_htmlspecialchars($user['title']);
	// If the user is banned
	else if (in_array(strtolower($user['username']), $ban_list, true))
		$user_title = $lang_common['Banned'];
	else if ($user['status'] <= PUN_USER)
	{
		// Are there any ranks? (> 1 because there is a dummy entry)
		if ($pun_config['o_ranks'] == '1' && count($rank_list) > 1)
		{
			@reset($rank_list);
			next($rank_list);
			while (list(, $value) = @each($rank_list))
			{
				if (intval($user['num_posts']) >= $value[1])
					$user_title = pun_htmlspecialchars($value[0]);
			}
		}

		// If the user didn't "reach" any rank
		if ($user_title == '')
			$user_title = $lang_common['Member'];
	}
	else if ($user['status'] == PUN_MOD)
		$user_title = $lang_common['Moderator'];
	else
		$user_title = $lang_common['Administrator'];

	return $user_title;
}


//
// Generate a string with numbered links (for multipage scripts)
//
function paginate($num_pages, $cur_page, $base_url)
{
	if ($num_pages <= 1)
		$string = '<u>1</u>';
	else
	{
		if ($cur_page > 4)
			$string = '<a href="'.$base_url.'&amp;p=1">1</a>&nbsp;...';

		// Don't ask me how the following works. It just does, OK? :-)
		for ($current = $cur_page - 3, $stop = $cur_page + 4; $current < $stop; ++$current)
		{
			if ($current < 1 || $current > $num_pages)
				continue;
			else if ($current != $cur_page)
				$string .= '&nbsp;<a href="'.$base_url.'&amp;p='.$current.'">'.$current.'</a>';
			else
				$string .= '&nbsp;<b>'.$current.'</b>';
		}

		if ($cur_page < ($num_pages-3))
			$string .= '&nbsp;...&nbsp;<a href="'.$base_url.'&amp;p='.$num_pages.'">'.$num_pages.'</a>';
	}

	return $string;
}


//
// Display a message
//
function message($message, $no_back_link = false)
{
	global $db, $pun_root, $lang_common, $pun_config, $pun_start;

	if (!defined('PUN_HEADER'))
	{
		global $cookie, $cur_user;

		$page_title = pun_htmlspecialchars($pun_config['o_board_title']).' / '.$lang_common['Info'];
		require $pun_root.'header.php';
	}

?>
<table class="punspacer" cellspacing="1" cellpadding="4"><tr><td>&nbsp;</td></tr></table>

<table class="punmain" cellspacing="1" cellpadding="4">
	<tr class="punhead">
		<td class="punhead"><?php echo $lang_common['Info'] ?></td>
	</tr>
	<tr>
		<td class="puncon1">
			<?php echo $message ?><br><br>
<?php if (!$no_back_link): ?>			<a href="JavaScript: history.go(-1)"><?php echo $lang_common['Go back'] ?></a>.
<?php endif; ?>		</td>
	</tr>
</table>

<table class="punspacer" cellspacing="1" cellpadding="4"><tr><td>&nbsp;</td></tr></table>
<?php

	require $pun_root.'footer.php';
}


//
// Display a message when board is in maintenance mode
//
function maintenance_message()
{
	global $pun_root, $pun_config, $lang_common, $cur_user;

	// Deal with newlines, tabs and multiple spaces
	$pattern = array("\n", "\t", '  ', '  ');
	$replace = array('<br>', '&nbsp; &nbsp; ', '&nbsp; ', ' &nbsp;');
	$message = str_replace($pattern, $replace, $pun_config['o_maintenance_message']);

	$style = (!empty($cur_user)) ? $cur_user['style'] : $pun_config['o_default_style'];


	// Load the maintenance template
	$fp = fopen($pun_root.'include/template/maintenance.tpl', 'r');
	$tpl_maint = trim(fread($fp, filesize($pun_root.'include/template/maintenance.tpl')));
	fclose($fp);


	// START SUBST - <pun_content_direction>
	$tpl_maint = str_replace('<pun_content_direction>', $lang_common['lang_direction'], $tpl_maint);
	// END SUBST - <pun_content_direction>


	// START SUBST - <pun_char_encoding>
	$tpl_maint = str_replace('<pun_char_encoding>', $lang_common['lang_encoding'], $tpl_maint);
	// END SUBST - <pun_char_encoding>


	// START SUBST - <pun_head>
	ob_start();

?>
<title><?php echo pun_htmlspecialchars($pun_config['o_board_title']).' / '.$lang_common['Maintenance'] ?></title>
<link rel="stylesheet" type="text/css" href="style/<?php echo $style.'.css' ?>">
<?php

	$tpl_temp = trim(ob_get_contents());
	$tpl_maint = str_replace('<pun_head>', $tpl_temp, $tpl_maint);
	ob_end_clean();
	// END SUBST - <pun_head>


	// START SUBST - <pun_maint_heading>
	$tpl_maint = str_replace('<pun_maint_heading>', $lang_common['Maintenance'], $tpl_maint);
	// END SUBST - <pun_maint_heading>


	// START SUBST - <pun_maint_message>
	$tpl_maint = str_replace('<pun_maint_message>', $message, $tpl_maint);
	// END SUBST - <pun_maint_message>


	// START SUBST - <pun_include "*">
	while (preg_match('<pun_include "(.*?)">', $tpl_maint, $cur_include))
	{
		ob_start();
		include($cur_include[1]);
		$tpl_temp = ob_get_contents();
		$tpl_maint = str_replace('<'.$cur_include[0].'>', $tpl_temp, $tpl_maint);
	    ob_end_clean();
	}
	// END SUBST - <pun_include "*">


	exit($tpl_maint);
}


//
// Display $message and redirect user to $destination
//
function redirect($destination, $message)
{
	global $pun_root, $pun_config, $lang_common, $cur_user;

	if ($destination == '')
		$destination = 'index.php';

	$style = (!empty($cur_user)) ? $cur_user['style'] : $pun_config['o_default_style'];


	// Load the redirect template
	$fp = fopen($pun_root.'include/template/redirect.tpl', 'r');
	$tpl_redir = trim(fread($fp, filesize($pun_root.'include/template/redirect.tpl')));
	fclose($fp);


	// START SUBST - <pun_content_direction>
	$tpl_redir = str_replace('<pun_content_direction>', $lang_common['lang_direction'], $tpl_redir);
	// END SUBST - <pun_content_direction>


	// START SUBST - <pun_char_encoding>
	$tpl_redir = str_replace('<pun_char_encoding>', $lang_common['lang_encoding'], $tpl_redir);
	// END SUBST - <pun_char_encoding>


	// START SUBST - <pun_head>
	ob_start();

?>
<meta http-equiv="refresh" content="<?php echo $pun_config['o_redirect_delay'] ?>;URL=<?php echo $destination ?>">
<title><?php echo pun_htmlspecialchars($pun_config['o_board_title']).' / '.$lang_common['Redirecting'] ?></title>
<link rel="stylesheet" type="text/css" href="style/<?php echo $style.'.css' ?>">
<?php

	$tpl_temp = trim(ob_get_contents());
	$tpl_redir = str_replace('<pun_head>', $tpl_temp, $tpl_redir);
	ob_end_clean();
	// END SUBST - <pun_head>


	// START SUBST - <pun_redir_heading>
	$tpl_redir = str_replace('<pun_redir_heading>', $lang_common['Redirecting'], $tpl_redir);
	// END SUBST - <pun_redir_heading>


	// START SUBST - <pun_redir_text>
	$tpl_temp = $message.'<br><br>'.'<a href="'.$destination.'">'.$lang_common['Click redirect'].'</a>';
	$tpl_redir = str_replace('<pun_redir_text>', $tpl_temp, $tpl_redir);
	// END SUBST - <pun_redir_text>


	// START SUBST - <pun_include "*">
	while (preg_match('<pun_include "(.*?)">', $tpl_redir, $cur_include))
	{
		ob_start();
		include($cur_include[1]);
		$tpl_temp = ob_get_contents();
		$tpl_redir = str_replace('<'.$cur_include[0].'>', $tpl_temp, $tpl_redir);
	    ob_end_clean();
	}
	// END SUBST - <pun_include "*">

	exit($tpl_redir);
}


//
// Format a time string according to $time_format and timezones
//
function format_time($timestamp, $date_only = false)
{
	global $pun_config, $lang_common, $cur_user;

	if ($timestamp == '')
		return $lang_common['Never'];

	if (!isset($cur_user) || $pun_config['o_server_timezone'] == $cur_user['timezone'])
		$diff = 0;
	else
		$diff = ($cur_user['timezone'] - $pun_config['o_server_timezone'])*3600;

	$timestamp += $diff;
	$now = time();

	$date = date($pun_config['o_date_format'], $timestamp);
	$today = date($pun_config['o_date_format'], $now+$diff);
	$yesterday = date($pun_config['o_date_format'], $now+$diff-86400);

	if ($date == $today)
		$date = $lang_common['Today'];
	else if ($date == $yesterday)
		$date = $lang_common['Yesterday'];

	if (!$date_only)
		return $date.' '.date($pun_config['o_time_format'], $timestamp);
	else
		return $date;
}


//
// Make sure that HTTP_REFERER matches $pun_config['o_base_url']/$script
//
function confirm_referer($script)
{
	global $pun_config, $lang_common;

	if (!preg_match('#^'.preg_quote(str_replace('www.', '', $pun_config['o_base_url']).'/'.$script, '#').'#i', str_replace('www.', '', $_SERVER['HTTP_REFERER'])))
		message($lang_common['Bad referer']);
}	


//
// Generate a random password of length $len
//
function random_pass($len)
{
	$chars = 'ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789';

	for ($i = 0; $i < $len; ++$i)
		$password .= substr($chars, (mt_rand() % strlen($chars)), 1); 

	return $password;
}


//
// Compute a hash of $str
// Uses sha1() if available. If not, mhash() if available. If not, it falls back on md5().
//
function pun_hash($str)
{
	if (function_exists('sha1'))	// Only in PHP 4.3.0+
		return sha1($str);
	else if (function_exists('mhash'))	// Only if Mhash library is loaded
		return bin2hex(mhash(MHASH_SHA1, $str));
	else
		return md5($str);
}


//
// Try to determine the correct remote IP-address
//
function get_remote_address()
{
	// If HTTP_X_FORWARDED_FOR is set we grab the first address in the list
	if (isset($_SERVER['HTTP_X_FORWARDED_FOR']))
	{
		if (preg_match('/[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}/', $_SERVER['HTTP_X_FORWARDED_FOR'], $addresses))
			return $addresses[0];
	}

	// If no address was found in HTTP_X_FORWARDED_FOR, we try HTTP_CLIENT_IP and if that isn't set we return REMOTE_ADDR
	return (isset($_SERVER['HTTP_CLIENT_IP'])) ? $_SERVER['HTTP_CLIENT_IP'] : $_SERVER['REMOTE_ADDR'];
}


//
// Add slashes only if magic_quotes_gpc is off
//
function escape($str)
{
	return (get_magic_quotes_gpc() == 1) ? $str : addslashes($str);
}


//
// Strip slashes only if magic_quotes_gpc is on
//
function unescape($str)
{
	return (get_magic_quotes_gpc() == 1) ? stripslashes($str) : $str;
}


//
// Equivalent to htmlspecialchars(), but allows &#[0-9]+ (for unicode)
//
function pun_htmlspecialchars($str)
{
	$str = preg_replace('/&(?!#[0-9]+;)/s', '&amp;', $str);
	$str = str_replace(array('<', '>', '"'), array('&lt;', '&gt;', '&quot;'), $str);

	return $str;
}


//
// Equivalent to strlen(), but counts &#[0-9]+ as one character (for unicode)
//
function pun_strlen($str)
{
	return strlen(preg_replace('/&#([0-9]+);/', '!', $str));
}


//
// Convert \r\n and \r to \n
//
function pun_linebreaks($str)
{
	return str_replace("\r", "\n", str_replace("\r\n", "\n", $str));
}


//
// Display a simple error message
//
function error($message, $file, $line, $db_error = false)
{
	global $pun_config;

	// Set a default title if the script failed before $pun_config could be populated
	if (empty($pun_config))
		$pun_config['o_board_title'] = 'PunBB';

	// Empty output buffer and stop buffering
	ob_end_clean();

	// "Restart" output buffering if we are using ob_gzhandler (since the gzip header is already sent)
	if ($pun_config['o_gzip'] == '1' && extension_loaded('zlib') && (strpos($_SERVER['HTTP_ACCEPT_ENCODING'], 'gzip') !== false || strpos($_SERVER['HTTP_ACCEPT_ENCODING'], 'deflate') !== false))
		ob_start('ob_gzhandler');

?>
<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.01 Transitional//EN" "http://www.w3.org/TR/html4/loose.dtd">

<html>
<head>
<meta http-equiv="Content-Type" content="text/html; charset=iso-8859-1">
<title><?php echo pun_htmlspecialchars($pun_config['o_board_title']) ?> / Error</title>
</head>
<body>

<br><br><br><br>

<table style="width: 60%; margin: auto; border: none; background-color: #666666" cellspacing="1" cellpadding="4">
	<tr>
		<td style="font: bold 10px Verdana, Arial, Helvetica, sans-serif; color: #593909; background-color: #EFAF50">An error was encountered</td>
	</tr>
	<tr>
		<td style="font: 10px Verdana, Arial, Helvetica, sans-serif; background-color: #DEDFDF">
<?php

	if (defined('PUN_DEBUG'))
	{
		echo "\t\t\t".'<b>File:</b> '.$file.'<br>'."\n\t\t\t".'<b>Line:</b> '.$line.'<br><br>'."\n\t\t\t".'<b>PunBB reported</b>: '.$message."\n";

		if ($db_error)
			echo "\t\t\t".'<br><b>Database reported:</b> '.htmlspecialchars($db_error['error']).' (Errno: '.$db_error['errno'].')'."\n";
	}
	else
		echo "\t\t\t".'Error: <b>'.$message.'.</b>'."\n";

?>
		</td>
	</tr>
</table>

</body>
</html>
<?php

	// If a database connection was established (before this error) we close it
	if ($db_error)
		$GLOBALS[db]->close();

	exit;
}


// DEBUG FUNCTIONS BELOW

//
// Dump contents of variable(s)
//
function dump($var1, $var2 = null)
{
	echo '<pre>';
	print_r($var1);

	if ($var2 != null)
	{
		echo "\n\n";
		print_r($var2);
	}

	echo '</pre>';
	exit;
}
